{***************************************************************
 *
 * Unit Name : FileOperation
 * Purpose   : Win32 File Operation
 * Author    : Eugen Mihailescu
 * Company   : EM Quicksoft Romania SRL
 * Copyright : 1998,2002  All rights reserved.
 * Web Site  : http://www.em-quicksoft.com
 * Support   : support@em-quicksoft.com
 * History   : 03/18/2000
 *
 * Disclaimer: The objectives of this program is only educational.
 *             You MUST NOT use it without getting any material benefit on it
 *             You have the right to share this code with others without removing
 *             the copyright and/or disclaimer notice.
 *             You are allowed to include its code in your application
 *             only if it was designed for non commercial purpose,
 *             otherwise we claim $10 for each copy of your
 *             application which contains this code inside.
 ****************************************************************}

Unit FileOperation;

Interface

Uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  Shellapi,
  ComCtrls;

Type
  TFileOperationType = (foNone, foCopy, foMove, foRename, foDelete, foRecycle);
  TWhatIsDoing = (isReading, isWritting, isNone, isMoving, isDeleting);
  TLanguage = (Romanian, English);
  TOnProgress = Procedure(Sender: TObject; PercComplete: Integer) Of Object;
  TFileOperation = Class(TComponent)
  private
    fSource: TFileName;
    fDestination: TFileName;
    fOpType: TFileOperationType;
    FLanguage: TLanguage;
    fProgress: TOnProgress;
    fTitle: String;
    FCaption: String;
    fROMMsg: TStringList;
    fENGMsg: TStringList;
    fOverwrite: Boolean;
  protected
    Function fCopyFile: Boolean;
    Function fRenameFile: Boolean;
    Function fMoveFile: Boolean;
    Function fDeleteFile: Boolean;
    Function fSendToRecycleBin: Boolean;
    Function fGetTitle(FLanguage: TLanguage; fOpType: TFileOperationType): String;
    Function fGetFrom(FLanguage: TLanguage): String;
    Function fGetTo(FLanguage: TLanguage): String;
    Function fGetEstimated(FLanguage: TLanguage; vTime: String): String;
    Function myCopyFile(vSource: String; vDestination: String; AllowOverwrite: Boolean): Boolean;
    Procedure SetROMMsg(Value: TStringList);
    Procedure SetENGMsg(Value: TStringList);
  public
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
    Function Execute: Boolean;
  published
    Property Source: TFileName read fSource write fSource;
    Property Destination: TFileName read fDestination write fDestination;
    Property Operation: TFileOperationType read fOpType write fOpType default foNone;
    Property Language: TLanguage read FLanguage write FLanguage default Romanian;
    Property OnProgress: TOnProgress read fProgress write fProgress;
    Property RomanianMsg: TStringList read fROMMsg write SetROMMsg;
    Property EnglishMsg: TStringList read fENGMsg write SetENGMsg;
    Property Overwrite: Boolean read fOverwrite write fOverwrite default False;
  End;
Procedure Register;

Implementation
uses FileOpForm;
//******************* TFileOperation.Create *************************

Constructor TFileOperation.Create(AOwner: TComponent);
Var
  I                 : Byte;
Begin
  Inherited Create(AOwner);
  fROMMsg := TStringList.Create;
  fENGMsg := TStringList.Create;
  fROMMsg.Add('Copiere fisiere');
  fENGMsg.Add('Copying files');
  fROMMsg.Add('Mutere fisiere');
  fENGMsg.Add('Moving files');
  fROMMsg.Add('Redenumire fisiere');
  fENGMsg.Add('Renaming files');
  fROMMsg.Add('Stergere fisiere');
  fENGMsg.Add('Deleting files');
  fROMMsg.Add('Golire Recycle Bin');
  fENGMsg.Add('Emptying Recycle Bin');
  fROMMsg.Add('De la');
  fENGMsg.Add('From');
  fROMMsg.Add('Catre');
  fENGMsg.Add('To');
  fROMMsg.Add('Timp estimat ramas');
  fENGMsg.Add('Estimated time left');
  frmFileOp := TfrmFileOp.Create(Application);
  fTitle := 'Copying';
  FCaption := 'Waiting to process...';
End;
//******************* TFileOperation.Destroy *************************

Destructor TFileOperation.Destroy;

Begin
  fROMMsg.Free;
  fENGMsg.Free;
  //  frmFileOp.Free;
  Inherited Destroy;
End;
//******************* TFileOperation.Execute *************************

Function TFileOperation.Execute: Boolean;
Begin
  Result := False;
  frmFileOp.vCanClose := False;
  frmFileOp.Caption := fTitle;
  frmFileOp.vMessage := 'Sunteti sigur ca doriti oprirea acestei operatii?';
  If FLanguage = Romanian Then frmFileOp.vMessage := 'Sunteti sigur ca doriti oprirea acestei operatii?';
  If FLanguage = English Then frmFileOp.vMessage := 'Are you sure you want to abort this process?';
  Case fOpType Of
    foCopy: Result := fCopyFile;
    foMove: Result := fMoveFile;
    foRename: Result := fRenameFile;
    foDelete: Result := fDeleteFile;
    foRecycle: Result := fSendtoRecycleBin;
    Else
      Result := False;
  End;
  frmFileOp.vCanClose := False;
  frmFileOp.vResult := False;
  frmFileOp.hide;
End;
//******************* TFileOperation.fCopyFile *************************

Function TFileOperation.fCopyFile: Boolean;
Var
  vList             : TStringList;
  I                 : Integer;
Begin
  vList := TStringList.Create;
  Try
    vList.CommaText := fSource;
    For I := 1 To vList.Count Do
      Begin
        frmFileOp.aviFileOp.CommonAVI := aviCopyFile;
        frmFileOp.aviFileOp.Active := True;
        frmFileOp.Caption := fGetTitle(FLanguage, fOpType);
        frmFileOp.lblFrom.Caption := fGetFrom(FLanguage);
        frmFileOp.lblTo.Caption := fGetTo(FLanguage);
        frmFileOp.Show;
        frmFileOp.Update;
        Result := myCopyFile(fSource, fDestination, fOverwrite);
        frmFileOp.aviFileOp.Active := False;
      End;
  Finally
    vList.Free;
  End;
End;
//******************* TFileOperation.fRenameFile *************************

Function TFileOperation.fRenameFile: Boolean;
Begin
  frmFileOp.aviFileOp.CommonAVI := aviCopyFiles;
  frmFileOp.aviFileOp.Active := True;
  frmFileOp.Caption := fGetTitle(FLanguage, fOpType);
  frmFileOp.lblFrom.Caption := fGetFrom(FLanguage);
  frmFileOp.lblTo.Caption := fGetTo(FLanguage);
  frmFileOp.Show;
  frmFileOp.Update;
  Result := RenameFile(fSource, fDestination);
  frmFileOp.aviFileOp.Active := False;
End;
//******************* TFileOperation.fMoveFile *************************

Function TFileOperation.fMoveFile: Boolean;
Begin
  frmFileOp.aviFileOp.CommonAVI := aviCopyFiles;
  frmFileOp.aviFileOp.Active := True;
  frmFileOp.Caption := fGetTitle(FLanguage, fOpType);
  frmFileOp.lblFrom.Caption := fGetFrom(FLanguage);
  frmFileOp.lblTo.Caption := fGetTo(FLanguage);
  frmFileOp.Show;
  frmFileOp.Update;
  If myCopyFile(fSource, fDestination, fOverwrite) Then
    Result := DeleteFile(fSource);
  frmFileOp.aviFileOp.Active := False;
End;
//******************* TFileOperation.fDeleteFile *************************

Function TFileOperation.fDeleteFile: Boolean;
Begin
  frmFileOp.aviFileOp.CommonAVI := aviDeleteFile;
  frmFileOp.aviFileOp.Active := True;
  frmFileOp.Caption := fGetTitle(FLanguage, fOpType);
  frmFileOp.lblFrom.Caption := fGetFrom(FLanguage);
  frmFileOp.lblTo.Caption := '';
  frmFileOp.Show;
  frmFileOp.Update;
  Result := DeleteFile(fSource);
  frmFileOp.aviFileOp.Active := False;
End;
//******************* TFileOperation.fSendToRecycleBin *************************

Function TFileOperation.fSendToRecycleBin: Boolean;
Begin
  frmFileOp.aviFileOp.CommonAVI := aviEmptyRecycle;
  frmFileOp.aviFileOp.Active := True;
  frmFileOp.Caption := fGetTitle(FLanguage, fOpType);
  frmFileOp.lblFrom.Caption := fGetFrom(FLanguage);
  frmFileOp.lblTo.Caption := '';
  frmFileOp.Show;
  frmFileOp.Update;
  Result := False;
  frmFileOp.aviFileOp.Active := False;
End;
//******************* TFileOperation.fGetTitle *************************

Function TFileOperation.fGetTitle(FLanguage: TLanguage; fOpType: TFileOperationType): String;
Var
  L                 : TStringList;
Begin
  If FLanguage = Romanian Then L := fROMMsg;
  If FLanguage = English Then L := fENGMsg;
  Case fOpType Of
    foCopy:
      If L.Count >= 0 Then
        Result := L.Strings[0]
      Else
        Result := '';
    foMove:
      If L.Count >= 1 Then
        Result := L.Strings[1]
      Else
        Result := '';
    foRename:
      If L.Count >= 2 Then
        Result := L.Strings[2]
      Else
        Result := '';
    foDelete:
      If L.Count >= 3 Then
        Result := L.Strings[3]
      Else
        Result := '';
    foRecycle:
      If L.Count >= 4 Then
        Result := L.Strings[4]
      Else
        Result := '';
    Else
      Result := '';
  End;
End;
//******************* TFileOperation.fGetFrom *************************

Function TFileOperation.fGetFrom(FLanguage: TLanguage): String;
Var
  L                 : TStringList;
Begin
  If FLanguage = Romanian Then L := fROMMsg;
  If FLanguage = English Then L := fENGMsg;
  Result := L.Strings[5] + ': ' + fSource;
End;
//******************* TFileOperation.fGetTo *************************

Function TFileOperation.fGetTo(FLanguage: TLanguage): String;
Var
  L                 : TStringList;
Begin
  If FLanguage = Romanian Then L := fROMMsg;
  If FLanguage = English Then L := fENGMsg;
  Result := L.Strings[6] + ': ' + fDestination;
End;
//******************* TFileOperation.fGetEstimated *************************

Function TFileOperation.fGetEstimated(FLanguage: TLanguage; vTime: String): String;
Var
  L                 : TStringList;
Begin
  If FLanguage = Romanian Then L := fROMMsg;
  If FLanguage = English Then L := fENGMsg;
  Result := L.Strings[7] + ': ' + vTime;
End;
//******************* TFileOperation.myCopyFile *************************

Function TFileOperation.myCopyFile(vSource: String; vDestination: String; AllowOverwrite: Boolean): Boolean;
Var
  FromF, ToF        : File;
  NumRead, NumWritten: Integer;
  Buf               : Array[1..2048] Of char;
  Position          : Integer;
  start             : TDateTime;
  vMessage          : String;
  vMessage1         : String;
Begin
  frmFileOp.lblEstimated.Caption := fGetEstimated(FLanguage, '00:00:00');
  vMessage := Format('Fisierul %-s deja exista. Doriti sa-l suprascieti?', [vDestination]);
  If FLanguage = Romanian Then
    vMessage := Format('Fisierul %-s deja exista. Doriti sa-l suprascieti?', [vDestination]);
  If FLanguage = English Then
    vMessage := Format('File %-s already exists. Are you sure you want to overwrite it?', [vDestination]);
  vMessage1 := Format('Fisierul %-s nu exista. ', [vSource]);
  If FLanguage = Romanian Then
    vMessage1 := Format('Fisierul %-s nu exista. ', [vSource]);
  If FLanguage = English Then
    vMessage1 := Format('File %-s does not exists.', [vSource]);
  If Not fileexists(vSource) Then
    Begin
      MessageDlg(vMessage1, mtError, [mbOK], 0);
      Result := False;
      Exit;
    End;
  If Not AllowOverwrite Then
    If fileexists(vDestination) Then
      If MessageDlg(vMessage, mtConfirmation, [mbYes, mbNo], 0) = mrNo Then Exit;
  start := Time;
  Try
    AssignFile(FromF, fSource);
    Reset(FromF, 1);
    AssignFile(ToF, fDestination);
    Rewrite(ToF, 1);
    Position := 0;
    Repeat
      If frmFileOp.progressbar.Position > 0 Then
        frmFileOp.lblEstimated.Caption := fGetEstimated(FLanguage, FormatDateTime('hh:nn:ss', (Time - start) *
          100 / frmFileOp.progressbar.Position - (Time - start)));
      If FileSize(fromf) > 0 Then
        Position := Trunc(100 * FileSize(tof) / FileSize(fromf));
      If Assigned(fProgress) Then
        fProgress(Nil, Position);
      frmFileOp.ProgressBar.Position := Position;
      Application.ProcessMessages;
      BlockRead(FromF, Buf, SizeOf(Buf), NumRead);
      BlockWrite(ToF, Buf, NumRead, NumWritten);
    Until (NumRead = 0) Or (NumWritten <> NumRead) Or (frmFileOp.vResult);
    CloseFile(FromF);
    CloseFile(ToF);
    Result := True;
  Except
    Result := False;
  End;
End;
//******************* TFileOperation.SetROMMsg *************************

Procedure TFileOperation.SetROMMsg(Value: TStringList);
Begin
  fROMMsg.CommaText := Value.CommaText;
End;
//******************* TFileOperation.SetENGMsg *************************

Procedure TFileOperation.SetENGMsg(Value: TStringList);
Begin
  fENGMsg.CommaText := Value.CommaText;
End;

Procedure Register;
Begin
  RegisterComponents('EM-Quicksoft', [TFileOperation]);
End;

End.

